package com.jpsoft.employment.config;

import com.jpsoft.employment.interceptor.LoginInterceptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.*;

@Configuration
public class WebMvcConfig implements WebMvcConfigurer {
	@Autowired
	private LoginInterceptor loginInterceptor;

	@Override
	public void addCorsMappings(CorsRegistry registry) {
		registry.addMapping("/**")
				.allowedOrigins("*")
				.allowedHeaders("*")
				.allowedMethods("*")
				.allowCredentials(false)
				.exposedHeaders("access-control-allow-headers",
						"access-control-allow-methods",
						"access-control-allow-origin",
						"access-control-max-age",
						"X-Frame-Options",
						"token-status")
				.maxAge(3600);
	}

	@Override
	public void addResourceHandlers(ResourceHandlerRegistry registry) {
		registry.addResourceHandler("/static/**").addResourceLocations("classpath:/static/");

		registry.addResourceHandler("swagger-ui.html")
				.addResourceLocations("classpath:/META-INF/resources/");

		registry.addResourceHandler("/webjars/**")
				.addResourceLocations("classpath:/META-INF/resources/webjars/");
	}

	@Override
	public void addViewControllers(ViewControllerRegistry registry) {
		// registry.addViewController("/login").setViewName("login");
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(loginInterceptor)
				.addPathPatterns("/**")
                .excludePathPatterns("/login",
						"/swagger-resources/**",
                        "/webjars/**",
                        "/swagger-ui.html",
						"/doc.html",
                        "/v2/**",
                        "/mobileApi/**")
                .excludePathPatterns("/wechat/**")
				.excludePathPatterns("/mobile/jobUserApi/findByOpenId")
				.excludePathPatterns("/mobile/jobUserApi/wechatLogin")
				.excludePathPatterns("/mobile/jobUserApi/validateCode")
				.excludePathPatterns("/mobile/jobUserApi/getVerifyCode")
				.excludePathPatterns("/mobile/jobUserApi/passwordLogin")
				.excludePathPatterns("/mobile/recruitmentApi/getRecruitmentList")
				.excludePathPatterns("/mobile/recruitmentApi/getScreenList")
				.excludePathPatterns("/mobile/sysUserApi/wechatLogin")
				.excludePathPatterns("/mobile/sysUserApi/findByOpenId")
				.excludePathPatterns("/mobile/sysUserApi/passwordLogin")
				.excludePathPatterns("/mobile/dataScreenApi/jobFairScreenData")
				.excludePathPatterns("/mobile/dataScreenApi/employmentOnlineData")
				.excludePathPatterns("/mobile/dataScreenApi/companyData")
				.excludePathPatterns("/mobile/dataScreenApi/recruitmentData")
				.excludePathPatterns("/mobile/dataScreenApi/seatData")
				.excludePathPatterns("/mobile/dataScreenApi/resumeDeliverData")
		;


	}
}
